BEGIN;

	CREATE MATERIALIZED VIEW IF NOT EXISTS org_features_view AS
		WITH products_custom_attrs AS (
			SELECT DISTINCT ON (E.org_key, E.product_id, E.system)
				E.org_key,
				E.product_id,
				E.system,
				E.updated_at,
				E.custom_attr
			FROM entitlements E
			WHERE
				E.is_deleted = false
				AND E.status = 'ACTIVE'::status
			ORDER BY E.org_key, E.product_id, E.system, E.updated_at DESC
		),
		features_custom_attrs AS (
			SELECT
				PCA.org_key,
				PF.feature_id,
				PF.system,
				jsonb_object_agg(FCA.attr, PCA.custom_attr -> FCA.attr) AS custom_attr
			FROM products_custom_attrs PCA
			LEFT JOIN product_features PF
				ON PF.product_id = PCA.product_id
					AND PF.system = PCA.system
			LEFT JOIN feature_custom_attrs FCA
				ON FCA.feature_id = PF.feature_id
			WHERE jsonb_typeof(PCA.custom_attr -> FCA.attr) != 'null'
			GROUP BY 1, 2, 3
		),
		features_window AS (
			SELECT
				E.created_at,
				E.updated_at,
				E.org_key,
				F.feature_id,
				F.internal_name,
				F.external_name,
				F.ui_name,
				F.ui_abbr,
				E.licenses,
				E.begin_date,
				E.end_date,
				jsonb_build_object(
					'created_at', E.created_at,
					'created_by', E.created_by,
					'updated_at', E.updated_at,
					'updated_by', E.updated_by,
					'entitlement_id', E.entitlement_id,
					'org_uuid', E.org_uuid,
					'org_key', E.org_key,
					'external_id', E.external_id,
					'product_id', E.product_id,
					'origin', E.origin,
					'system', E.system,
					'status', E.status,
					'begin_date', E.begin_date,
					'end_date', E.end_date,
					'licenses', E.licenses,
					'eval', E.eval,
					'config', E.config,
					'custom_attr', E.custom_attr
				) AS entitlement_json,
				COALESCE(FCA.custom_attr, '{}'::jsonb) AS "custom_attr",
				CASE
					WHEN
						COALESCE(E.end_date, 'now'::timestamp + '1 day'::interval) <= 'now'::timestamp
					THEN 0
					WHEN
						COALESCE(E.begin_date, 'now'::timestamp + '1 day'::interval)	> 'now'::timestamp
						AND COALESCE(E.end_date, 'now'::timestamp + '2 day'::interval) > 'now'::timestamp
					THEN 1
					WHEN
						COALESCE(E.begin_date, 'now'::timestamp + '1 day'::interval) <= 'now'::timestamp
						AND COALESCE(E.end_date, 'now'::timestamp + '2 day'::interval) > 'now'::timestamp
					THEN 2
				END AS "window"
			FROM entitlements E
			JOIN products P
				ON P.product_id = E.product_id
					AND P.system = E.system
			JOIN product_features PF
				ON PF.product_id = P.product_id
					AND PF.system = P.system
			JOIN features F
				ON F.feature_id = PF.feature_id
			LEFT JOIN features_custom_attrs FCA
				ON FCA.org_key = E.org_key
					AND FCA.feature_id = PF.feature_id
					AND FCA.system = PF.system
			WHERE
				E.is_deleted = false
				AND E.status = 'ACTIVE'::status
		),
		org_features AS (
			SELECT * FROM (
				SELECT
					FW.org_key,
					FW.feature_id,
					FW.internal_name,
					FW.external_name,
					FW.ui_name,
					FW.ui_abbr,
					FW.custom_attr,
					jsonb_agg(FW.entitlement_json) OVER (
						PARTITION BY FW.org_key, FW.feature_id
						ORDER BY FW.created_at DESC, FW.updated_at DESC
						RANGE BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING
					) AS entitlements,
					SUM(FW.licenses) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS licenses,
					MIN(FW.begin_date) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS begin_date,
					MAX(FW.end_date) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS end_date,
					FW.window,
					ROW_NUMBER() OVER (PARTITION BY FW.org_key, FW.feature_id ORDER BY FW.window DESC) AS row_number
				FROM features_window FW
			) FP
			WHERE FP.row_number = 1
		)

		SELECT
			OF.org_key,
			OF.feature_id,
			OF.internal_name,
			OF.external_name,
			OF.ui_name,
			OF.ui_abbr,
			OF.entitlements,
			OF.custom_attr,
			OF.licenses,
			OF.begin_date,
			OF.end_date,
			CASE
				WHEN OF.window = 0 THEN 'COMPLETE'
				WHEN OF.window = 1 THEN 'PENDING'
				WHEN OF.window = 2 THEN 'ACTIVE'
			END AS "window"
		FROM org_features OF;

COMMIT;
