BEGIN;
	DROP MATERIALIZED VIEW IF EXISTS org_features_view;

	CREATE MATERIALIZED VIEW IF NOT EXISTS org_features_view AS
	WITH products_custom_attrs AS (
		SELECT DISTINCT ON (E.org_key, E.product_id, E.system)
			E.org_key,
			E.product_id,
			E.system,
			E.updated_at,
			E.custom_attr
		FROM entitlements E
		WHERE
			E.is_deleted = false
			AND E.status IN ('ACTIVE','COMPLETE')
		ORDER BY E.org_key, E.product_id, E.system, E.updated_at DESC
	),
	entitlements_json AS (
		SELECT
			E.entitlement_id,
			row_to_json(E.*) AS "entitlement_json"
		FROM (
			SELECT
				E.created_at,
				E.created_by,
				E.updated_at,
				E.updated_by,
				E.entitlement_id,
				E.org_uuid,
				E.org_key,
				E.external_id,
				E.product_id,
				E.origin,
				E.system,
				E.status,
				E.begin_date,
				E.end_date,
				E.licenses,
				E.eval,
				E.config,
				E.custom_attr
			FROM entitlements E
			WHERE E.is_deleted = false
				AND E.status IN ('ACTIVE','COMPLETE')
		) E
	),
	features_custom_attrs AS (
		SELECT
			PCA.org_key,
			PF.feature_id,
			PF.system,
			jsonb_object_agg(FCA.attr, PCA.custom_attr -> FCA.attr) AS custom_attr
		FROM products_custom_attrs PCA
		LEFT JOIN product_features PF
			ON PF.product_id = PCA.product_id
				AND PF.system = PCA.system
		LEFT JOIN feature_custom_attrs FCA
			ON FCA.feature_id = PF.feature_id
		WHERE jsonb_typeof(PCA.custom_attr -> FCA.attr) != 'null'
		GROUP BY 1, 2, 3
	),
	features_window AS (
		SELECT
			E.created_at,
			E.updated_at,
			E.org_key,
			F.feature_id,
			F.internal_name,
			F.external_name,
			F.ui_name,
			F.ui_abbr,
			COALESCE(FCA.custom_attr, '{}'::jsonb) AS "custom_attr",
			E.licenses,
			E.begin_date,
			CASE WHEN E.status = 'COMPLETE' THEN E.completed_at ELSE E.end_date END AS "end_date",
			EJ.entitlement_json,
			CASE
				WHEN
					COALESCE(CASE WHEN E.status = 'COMPLETE' THEN E.completed_at ELSE E.end_date END, 'now'::timestamp + '1 day'::interval) <= 'now'::timestamp
				THEN 0
				WHEN
					COALESCE(E.begin_date, 'now'::timestamp + '1 day'::interval) > 'now'::timestamp
					AND COALESCE(CASE WHEN E.status = 'COMPLETE' THEN E.completed_at ELSE E.end_date END, 'now'::timestamp + '2 day'::interval) > 'now'::timestamp
				THEN 1
				WHEN
					COALESCE(E.begin_date, 'now'::timestamp + '1 day'::interval) <= 'now'::timestamp
					AND COALESCE(CASE WHEN E.status = 'COMPLETE' THEN E.completed_at ELSE E.end_date END, 'now'::timestamp + '2 day'::interval) > 'now'::timestamp
				THEN 2
			END AS "window"
		FROM entitlements E
		JOIN entitlements_json EJ
			ON EJ.entitlement_id = E.entitlement_id
		JOIN products P
			ON P.product_id = E.product_id
				AND P.system = E.system
		JOIN product_features PF
			ON PF.product_id = P.product_id
				AND PF.system = P.system
		JOIN features F
			ON F.feature_id = PF.feature_id
		LEFT JOIN features_custom_attrs FCA
			ON FCA.org_key = E.org_key
				AND FCA.feature_id = PF.feature_id
				AND FCA.system = PF.system
		WHERE
			E.is_deleted = false
			AND E.status IN ('ACTIVE','COMPLETE')
	),
	org_features AS (
		SELECT * FROM (
			SELECT
				FW.org_key,
				FW.feature_id,
				FW.internal_name,
				FW.external_name,
				FW.ui_name,
				FW.ui_abbr,
				FW.custom_attr,
				jsonb_agg(FW.entitlement_json) OVER (
					PARTITION BY FW.org_key, FW.feature_id
					ORDER BY FW.created_at DESC, FW.updated_at DESC
					RANGE BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING
				) AS entitlements,
				SUM(FW.licenses) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS licenses,
				MIN(FW.begin_date) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS begin_date,
				MAX(FW.end_date) OVER (PARTITION BY FW.org_key, FW.feature_id, FW.window) AS end_date,
				FW.window,
				ROW_NUMBER() OVER (PARTITION BY FW.org_key, FW.feature_id ORDER BY FW.window DESC) AS row_number
				FROM features_window FW
		) FP
		WHERE FP.row_number = 1
	),
	org_max_end_date AS (
		SELECT
			E.org_key,
			MAX(CASE
				WHEN
					E.status = 'ACTIVE'
				THEN
					E.end_date
				ELSE
					E.completed_at
			END) AS max_end_date
		FROM entitlements E
		JOIN products P
		ON
			P.is_deleted = false
			AND P.product_id = E.product_id
			AND P.system = E.system
			AND P.type != 'SINGLETON'
		WHERE
			E.is_deleted = false
			AND E.status IN ('ACTIVE','COMPLETE')
		GROUP BY E.org_key
	),
	org_grace_period AS (
		SELECT
			OF.org_key,
			OF.custom_attr -> 'grace_period_days' AS grace_period_days
		FROM org_features OF
		WHERE OF.feature_id = 'psc:feature:graceperiod'
	),
	org_deregistration_status AS (
		SELECT
			D.org_key,
			D.status AS deregistration_status
		FROM deregistrations D
		WHERE
			D.is_deleted = false
			AND D.status != 'CANCEL'
	),
	org_grace_period_dates AS (
		SELECT
			OGP.org_key,
			OMED.max_end_date AS "begin_date",
			CASE
				WHEN
					OGP.grace_period_days IS NOT NULL
				THEN
					OMED.max_end_date + (OGP.grace_period_days::bigint * '1 day'::interval)
			END as "end_date"
		FROM org_grace_period OGP
		LEFT JOIN org_max_end_date OMED ON OMED.org_key = OGP.org_key
	),
	org_grace_period_window AS (
		SELECT
			OGPD.org_key,
			CASE
				WHEN
					ODS.deregistration_status IS NOT NULL
					OR COALESCE(OGPD.end_date, 'now'::timestamp + '1 day'::interval) < 'now'::timestamp
				THEN
					0 --COMPLETED
				WHEN
					COALESCE(OGPD.begin_date, 'now'::timestamp + '1 day'::interval) > 'now'::timestamp
				THEN
					1 --PENDING
				WHEN
					COALESCE(OGPD.begin_date, 'now'::timestamp + '1 day'::interval) <= 'now'::timestamp
					AND 'now'::timestamp <= COALESCE(OGPD.end_date, 'now'::timestamp + '1 day'::interval)
				THEN
					2 --ACTIVE
			END as "window"
		FROM org_grace_period_dates OGPD
		LEFT JOIN org_deregistration_status ODS ON ODS.org_key = OGPD.org_key
	),
	org_features_calculated AS (
		SELECT
			OF.org_key,
			OF.feature_id,
			OF.internal_name,
			OF.external_name,
			OF.ui_name,
			OF.ui_abbr,
			OF.custom_attr,
			OF.entitlements,
			OF.licenses,
			CASE
				WHEN OF.feature_id = 'psc:feature:graceperiod'
				THEN OGPD.begin_date
				ELSE OF.begin_date
			END AS begin_date,
			CASE
				WHEN OF.feature_id = 'psc:feature:graceperiod'
				THEN OGPD.end_date
				ELSE OF.end_date
			END AS end_date,
			CASE
				WHEN OF.feature_id = 'psc:feature:graceperiod'
				THEN OGPW.window
				ELSE OF.window
			END AS window
		FROM org_features OF
		LEFT JOIN org_grace_period_dates OGPD ON OGPD.org_key = OF.org_key
		LEFT JOIN org_grace_period_window OGPW ON OGPW.org_key = OF.org_key
	)

	SELECT
		OF.org_key,
		OF.feature_id,
		OF.internal_name,
		OF.external_name,
		OF.ui_name,
		OF.ui_abbr,
		OF.custom_attr,
		OF.entitlements,
		OF.licenses,
		OF.begin_date,
		OF.end_date,
		CASE
			WHEN OF.window = 0 THEN 'COMPLETE'
			WHEN OF.window = 1 THEN 'PENDING'
			WHEN OF.window = 2 THEN 'ACTIVE'
		END AS "window"
	FROM org_features_calculated OF;

COMMIT;
